<?php
class FormBuilder {
  //
  // convert a field in a database to a form
  //
  var $database;
  private $default = NULL;
  var $record;
  var $symbols;
  private $size = NULL;
  var $table;
  
  function __construct($database) {
    //
    // creates a new formBuilder
    //
    $this->database = $database;
    $this->symbols = array(
      "{AE}"      => "Æ",
      "{ae}"      => "æ",
      "{f}"       => "ƒ",
      "{oe}"      => "œ",
      "{s}"       => "ſ",
      "{F-inv}"   => "Ⅎ",
      "{a-inv}"   => "ɐ",
      "{circ}"    => html_entity_decode("&#0770;", ENT_NOQUOTES, "UTF-8"),
      "{til}"     => html_entity_decode("&#0771;", ENT_NOQUOTES, "UTF-8"),
      "{macron}"  => html_entity_decode("&#0772;", ENT_NOQUOTES, "UTF-8"),
      "{trema}"   => html_entity_decode("&#0776;", ENT_NOQUOTES, "UTF-8"),
      "{gancho}"  => html_entity_decode("&#0777;", ENT_NOQUOTES, "UTF-8"),
      "{virgula}" => html_entity_decode("&#0787;", ENT_NOQUOTES, "UTF-8"),
      "{anel}"    => html_entity_decode("&#0778;", ENT_NOQUOTES, "UTF-8"),
      "{agudo}"   => html_entity_decode("&#0769;", ENT_NOQUOTES, "UTF-8"),
      "{breve}"   => html_entity_decode("&#0774;", ENT_NOQUOTES, "UTF-8")
    );
  }

  function buildDataSent() {
    //
    // create a hiden input with name dataSent e value '1'
    //
    print("<input type=\"hidden\" name=\"dataSent\" id=\"dataSent\""
      . " value=\"1\" />");
  }

  function buildForeignInput($name, $url, $default) {
    //
    // builds a input for a foreign field in current table that can 
    // access parent table info 
    //
    $value = $this->record[$name];
    $size = $this->getSize($name);
    $javascript = "getForeignDetails(event, '$url')";
    print("<input type=\"text\" id=\"$name\" name=\"$name\" size=\"$size\""
      . " value=\"$value\" onkeypress=\"$javascript\""
      . " onblur=\"$javascript\"/>\n<span id=\"details\">$default</span> ");
  }
  
  function buildHidden($name) {
    //
    // builds a hide input
    //
    $value = $this->record[$name];
    print("<input type=\"hidden\" id=\"$name\" name=\"$name\""
      . " value=\"$value\" />\n");
  }
  
  function buildInput($name) {
    //
    // builds a input for a field in current table
    //
    $value = $this->record[$name];
    $size = $this->getSize($name); 
    print("<input type=\"text\" id=\"$name\" name=\"$name\" "
      . "size=\"$size\" value=\"$value\" />");
  }
  
  function buildLabel($name) {
    //
    // builds a label for a field in current table
    //
    $humanName = $this->table->fieldsByName[$name]->humanName; 
    print("<label for=\"$name\"> $humanName: </label>");
  }
  
  function buildNavigation($controller, $offset) {
    //
    // builds an navigation toolbar
    //
    if ($offset == NULL) {
      $offset = 0;
    }
    $total = $controller->count();
    $last = (int) ($total / $controller->sqlMax); 
    $begin = $offset * $controller->sqlMax + 1;
    $end = ($offset + 1) * $controller->sqlMax;
    if ($end > $total) {
      $end = $total;
    }
    print("<p>\n");
    if ($offset > 1) {
      $this->buildNavigationLink("&lt;&lt; Início", 0);
    }
    if ($offset > 0) {
      $this->buildNavigationLink("&lt; Voltar", $offset - 1);
    }
    print("$begin a $end de $total\n");
    if ($offset < $last) {
      $this->buildNavigationLink("Avançar &gt;", $offset + 1);
    }
    if ($offset < $last - 1) {
      $this->buildNavigationLink("Fim &gt;&gt;", $last);
    }
    print("</p>\n");
  }
  
  function buildNavigationLink($text, $offset) {
    //
    // builds an link to navigation bar
    //
      $link = $_SERVER["PHP_SELF"]; 
      print("<a href=\"$link?offset=$offset\">$text</a>\n");
    }
  
  function buildSelect($name, $values) {
    //
    // builds a select for a field in current table
    //
    $default = $this->record[$name];
    if ($default == NULL && $this->default != NULL) {
      $default = $this->default;
      $this->default == NULL;
    }
    $select = "<select name=\"$name\">\n";
    while ($value = mysql_fetch_array($values)) {
      $id = htmlspecialchars($value[0]);
      $description = htmlspecialchars($value[1]);
      if ($id != $default) {    
        $select .= "<option value=\"$id\">$description</option>\n";
      } else {
        $select .= "<option value=\"$id\" selected=\"selected\">"
          . "$description</option>\n";
      }
    }
    $select .= "</select>\n";   
    print($select);
  }

  function buildSelectBoolean($name, $vtrue, $vfalse, $default) {
    //
    // build a select with true and false
    //
    $select = "<select name=\"$name\">\n";
    if ($this->record[$name] != NULL) {
      if ($this->record[$name] == 1) {
        $default = true;
      } else {
        $default = false;
      }
    }
    if ($default == true) {
      $select .= 
        "<option value=\"0\" >$vfalse</option>\n";
      $select .= 
        "<option value=\"1\" selected=\"selected\">$vtrue</option>\n";
    } else {
      $select .= 
        "<option value=\"0\" selected=\"selected\">$vfalse</option>\n";
      $select .= 
        "<option value=\"1\" >$vtrue</option>\n";
    }
    $select .= "</select>\n";
    print($select);
  }
  
  function buildTableHidden() {
    //
    // builds a hidden input for a field the current table
    // also builds an input to mark if the record is deleted
    //
    $table = $this->table->name;
    print("<input type=\"hidden\" id=\"table\" name=\"table\"" . 
      " value=\"$table\" />\n");
    print("<input type=\"hidden\" id=\"is_deleted\" name=\"is_deleted\"" . 
      " value=\"false\" />\n");
  }
  
  function buildTextArea($name) {
    //
    // builds a textarea for the a field in current table
    //
    $rows = 2;
    $value = $this->record[$name];
    if ($value) {
      $rows = $this->sizeTextArea($value, 70);
      if ($rows < 2) {
        $rows = 2;
      }
    }
    print("<textarea id=\"$name\" name=\"$name\" rows=\"$rows\" cols=\"70\""
      . " onkeyup=\"procorph.sizeTextArea(this)\">$value</textarea>"); 
  }
  
  function buildUpDownRemove() {
    //
    // builds the buttons up, down and remove
    //
    print("<a href=\"#\" onclick=\"moveChild(event, -1)\">"
      . "Acima</a> /\n");
    print("<a href=\"#\" onclick=\"moveChild(event, 1)\">"
      . "Abaixo</a> /\n");
    print("<a href=\"#\" onclick=\"deleteChild(event)\">"
      . "Remover</a>\n"); 
  }

  function convertToUnicode($text) {
    //
    // convert text beetwen '{' and '}' to unicode according the convention
    //
    $beginSpecial = false;
    $buffer = "";
    $newText = "";
    $size = strlen($text);
    for ($i=0; $i<$size; $i++) {
      $character = substr($text, $i, 1);
      if ($character == "{") {
        if ($buffer != "") {
          $newText .= "{" . $buffer;
          $buffer = "";
        }
        $beginSpecial = true;
        continue;
      }
      if ($character == "}" && $beginSpecial) {
        $buffer = "{" . $buffer . "}";
        if ($this->symbols[$buffer]) {
          $newText .= $this->symbols[$buffer];
        } else {
          $newText .= $buffer;
        }
        $beginSpecial = false;
        $buffer = "";
        continue;
      }
      if ($beginSpecial) {
        $buffer .= $character;
      } else {
        $newText .= $character;
      }
    }
    if ($beginSpecial) {
      $newText .= "{" . $buffer;
    }
    return $newText;
  }
  
  function formatDate($date) {
    //
    // formats a date in brazilian format
    // mysql function date_format not used to keep selects simple
    //
    $day = substr($date, 8, 2);
    $month = substr($date, 5, 2);
    $year = substr($date, 0, 4);
    return "$day/$month/$year";
  }

  function formatDateTime($date) {
    //
    // formats a datetime in brazilian format
    // mysql function date_format not used to not complicate selects
    //
    $hour = substr($date, 11, 2);
		$minute = substr($date, 14, 2);
		$second = substr($date, 17, 2);
		$day = substr($date, 8, 2);
		$month = substr($date, 5, 2);
		$year = substr($date, 0, 4);
		return "$day/$month/$year às $hour:$minute:$second";
  }
  
	function formatText($text) {
	  //
	  // format a text in html through character convetions
	  // utf-8 compliant (only ascii bytes are replaced)
	  //
	  $beginStrong = false;
	  $beginEm = false;
	  $beginSup = false;
	  $beginEscape = false;
	  $newText = "";
	  $tamanho = strlen($text);
	  // formating
	  for($i=0; $i<$tamanho; $i++) {
	    $caracter = substr($text, $i,1);
	    // escape
	    if ($caracter == "\\" && ! $beginEscape) {
	      $beginEscape = true;
	      continue;
	    }
	    if (! $beginEscape) {
	      if ($caracter == "*") {
	        // strong
	        if (! $beginStrong) {
	          $caracter = "<strong>";
	        } else {
	          $caracter = "</strong>";
	        }
	        $beginStrong = ! $beginStrong;
	      } elseif ($caracter == "/") {
	        // em
	        if (! $beginEm) {
	          $caracter = "<em>";
	        } else {
	          $caracter = "</em>";
	        }
	        $beginEm = ! $beginEm;
	      } elseif ($caracter == "^" || $caracter == " ") {
	        // sup
	        if (! $beginSup && $caracter == "^") {
	          $caracter = "<sup>";
	          $beginSup = true;
	        } elseif ($beginSup && $caracter == " ") {
	          $caracter = "</sup> ";
	          $beginSup = false;
	        }
	      }
	    } else {
	      $beginEscape = false;
	    }
	    // generating new text
	    $newText .= $caracter;
	  }
	  if ($beginStrong) {
	    $newText .= "</strong>";
	  }
	  if ($beginEm) {
	    $newText .= "</em>";
	  }
	  if ($beginSup) {
	    $newText .= "</sup>";
	  }
	  return $newText;
	}
  
  function getFormTitle() {
    //
    // get the default form title based on table human name
    //
    return $this->table->humanName;
  }
  
  private function getSize($name) {
    //
    // get a default size for a control
    //
    if ($this->size) {
      $size = $this->size;
      $this->size = NULL;
      return $size;
    }
    $field = $this->table->fieldsByName[$name];
    if ($field->type == "int" || $field->type == "integer") {
      return 4;
    }
    return $field->size; 
  }
  
  function sanitizeHtml(&$record) {
    //
    // avoid Html injection in sql data with htmlspecialchars
    //
    foreach($record as $field => $value) {
      $record[$field] = htmlspecialchars($value);
    }
  }
  
  function setDefault($default) {
    $this->default = $default;
  }
  
  function setSize($size) {
    //
    // overwrite default control size
    //
    $this->size = $size;
  }
  
  function setTable($tableName) {
    //
    // defines the current table
    //
    $this->table = $this->database->tablesByName[$tableName];
  }
  
  function setRecord($record) {
    //
    // defines the current record
    //
    $this->record = $record;
    $this->sanitizeHtml($this->record);
  }
  
  function showHumanName($name) {
    //
    // shows the humanName of the current field 
    //
    print($this->table->fieldsByName[$name]->humanName);
  }
  
  function showValue ($name) {
    //
    // shows the value of the current field
    //
    print($this->record[$name]);
  }
  
  private function sizeTextArea($content, $columns) {
    //
    // calculates the height of a text area based on its content
    //
    $lines = split("\n", $content);
	  $height = 0;
	  foreach($lines as $line) {
	    $lenght = strlen($line);
	    if ($lenght > $columns) {
	      $height += floor($lenght/$columns);
	    }
	  }
	  $height += count($lines);
	  return $height;  
  }

}

?>
