<?php
// 
// this file generate phps, html and sql when runned
//
require("database.php");

class ModelLoader {
	//
	// auto generate automatically sql and stores database info  
	//
	var $database;
	
	function __construct() {
	   $this->database = new Database("procorph", "Procorph");
	   $this->loadAll();
	}
	
	function buildTables() {
		//
		// build sqls, phps and htmls
		//
		$message = "# Generated automatically by MvcBuilder.\n" .
			"# Do not change or edit\n";
		// generating sql creates
		print("# " . $this->database->name . ".sql\n");
    print($message);
    foreach($this->database->tables as $table) {
      print("\n# $table->humanName\n");
      print($table->sqlCreate() . "\n");
      foreach($table->populate as $record) {
        print($record . "\n");
      }
    }
	}
	
	function loadAll() {
		//
		// loads all tables
		//
		$this->loadGroups();
    $this->loadUsers();
    $this->loadClasses();
    $this->loadStatus();
    $this->loadEntries();
    $this->loadSpellings();
    $this->loadDefinitions();
    $this->loadRelateds();
    $this->loadNotes();
    $this->loadTexts();
    $this->loadConcordances();
    $this->loadGlossaries();
	}

  function loadClasses() {
    //
    // loads table classes
    //
    $this->database->addTable("classes", "Classes");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Acepção", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
  }

  function loadConcordances() {
    //
    // load table concordancies
    //
    $this->database->addTable("concordances", "Concordâncias");
    $table = $this->database->getLastTable();
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("word", "Palavra", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("complement", "complemento", "text", NULL);
  }

  function loadDefinitions() {
    //
    // loads table Definitions
    //
    $this->database->addTable("definitions", "Acepções");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("description", "Acepção", "text", NULL);
    $table->getLastField()->setIndexRegular(100);
    $table->addField("attributes", "Atributos", "varhcar", 100);
    $table->addField("example", "Abonação", "text", NULL);
    $table->getLastField()->setIndexRegular(100);
    $table->addField("text_id", "Texto", "varchar", 8);
    $table->addField("page", "Página", "decimal", 4);
    $table->addField("order_def", "Ordem", "integer", NULL);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("entry_id", "Verbete", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "entries", "id", "cascade", "restrict");
  }
    
  function loadEntries() {
    //
    // load table entries
    //
    $this->database->addTable("entries", "Verbetes");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Verbete", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("class_id", "Classe e atributo", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "classes", "id", "restrict", "restrict");
    $table->addField("status_id", "Situação", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "status", "id", "restrict", "restrict");
    $table->addField("creation_date", "Data de criação", "date", NULL);
    $table->addField("first_example", "Primeira datação", "text", NULL);
    $table->getLastField()->setIndexRegular(100);
    $table->addField("text_id", "Texto", "varchar", 8);
    $table->addField("page", "Página", "decimal", 4);
    $table->addField("user_id", "Redator", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "users", "id", "restrict", "restrict");
    $table->addField("parent", "Pai", "integer", NULL);
    $table->getLastField()->setNull(true);
    $table->getLastField()->setIndexForeign(
      "entries", "id", "cascade", "cascade");
  }
  
  function loadGlossaries() {
    //
    // load table know_spellings
    //
    $this->database->addTable("glossaries", "Glossários");
    $table = $this->database->getLastTable();
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("word1", "Palavra", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("word2", "Relacionada", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("category", "Categoria", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("extras", "Extras", "varchar", 100);
  }

  function loadGroups() {
    //
    // load table groups
    //
    $this->database->addTable("groups", "Grupos");
    $table = $this->database->getLastTable();
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Nome", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("level", "Nível", "integer", NULL);
  }

  function loadNotes() {
    //
    // load table Spellings
    //
    $this->database->addTable("notes", "Observações");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("description", "Observação", "text", NULL);
    $table->getLastField()->setIndexRegular(100);
    $table->addField("order_note", "Ordem", "integer", NULL);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("entry_id", "Verbete", "integer", NULL);
    $table->getLastField()->setIndexForeign("entries", "id", "cascade", "restrict");
  }

  function loadRelateds() {
    //
    // load table Relateds
    //
    $this->database->addTable("relateds", "Verbetes relacionados");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Verbete relacionado", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("order_related", "Ordem", "integer", NULL);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("entry_id", "Verbete", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "entries", "id", "cascade", "restrict");
  }

  function loadSpellings() {
    //
    // load table Spellings
    //
    $this->database->addTable("spellings", "Variantes de grafia");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "int", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Variante", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("order_spelling", "Ordem", "integer", NULL);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("entry_id", "Verbete", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "entries", "id", "cascade", "restrict");
  }
  
  function loadUsers() {
    //
    // load table user
    //
    $i = 0;
    $this->database->addTable("users", "Usuários");
    $table = $this->database->getLastTable();
    // fields
    $table->addField("id", "Id", "integer", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("login", "Login", "varchar", 30);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("name", "Nome", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("email", "E-mail", "varchar", 100);
    $table->addField("password", "Senha", "varchar", 50);
    $table->addField("last_login", "Último acesso", "datetime", NULL);
    $table->getLastField()->setNull(true);
    $table->addField("active", "Ativo", "boolean", NULL);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("group_id", "Grupo", "integer", NULL);
    $table->getLastField()->setIndexForeign(
      "groups", "id", "restrict", "restrict");
  }
  
  function loadStatus() {
    //
    // load table groups
    //
    $this->database->addTable("status", "Situações");
    $table = $this->database->getLastTable();
    $table->addField("id", "Id", "integer", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Nome", "varchar", 60);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("level", "Nível", "integer", NULL);
  }
  
  function loadTexts() {
    //
    // load table texts
    //
    $this->database->addTable("texts", "Textos");
    $table = $this->database->getLastTable();
    $table->addField("id", "Id", "integer", NULL);
    $table->getLastField()->setIndexPrimary(true);
    $table->addField("name", "Nome", "varchar", 8);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("title", "Título", "text", NULL);
    $table->getLastField()->setIndexRegular(100);
    $table->addField("author", "Autores", "varchar", 100);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("creation_year", "Ano de criação", "varchar", 11);
    $table->getLastField()->setIndexRegular(NULL);
    $table->addField("edition_year", "Ano de edição", "varchar", 11);
    $table->getLastField()->setIndexRegular(NULL);
  }
  
}

?>
