<?php
require("field.php");

class Table {
  //
  // stores sql tables information and allows sql creation 
  // 
  var $fields = array();
  var $fieldsByName = array();
  var $humanName = "";
  var $index = false;
  var $indexSize = 0;
  var $name;
  var $onDelete;
  var $onUpdate;
  var $primary = false;
  var $populate = array();
  var $totalFields = 0;
  
  private $hasSize = array(
    "char" => true, 
    "decimal" => true,
    "varchar" => true
  );
  
  function __construct($name, $humanName) {
    //
    // create a new table
    //
    $this->name = $name;
    $this->humanName = $humanName;
  }
  
  function addField($name, $humanName, $type, $size) {
    //
    // add a new field in this table
    //
    $field = new Field($name, $humanName, $type, $size); 
    $this->fields[$this->totalFields++] = $field;
    $this->fieldsByName[$name] = $field;
  }
  
  function getLastField() {
    //
    // return the last field added in this table
    //
    if (! $this->fields) {
      return NULL;
    }
    return ($this->fields[$this->totalFields - 1]);
  }

  function sqlCreate() {
    //
    // returns create table comand
    //
    $firstField = true;
    // step 0: table name 
    $create = "create table $this->name (";
    foreach ($this->fields as $field) {
      // step 1: field name, type and size
      if ($firstField) {
        $create .= $field->name;
        $firstField = false;
      } else {
        $create .= ", $field->name";
      }
      $create .= " $field->type";
      if ($this->hasSize["$field->type"]) {
        $create .= " ($field->size)";
      }
      // step 3: null, increment and keys
      if (! $field->isNull) {
        $create .= " not null";
      }
      if ($field->default != "") {
        $create .= " default " . $field->default;
      }
      if ($field->isAutoIncrement) {
        $create .= " auto_increment";
      }
      if ($field->indexPrimary) {
        $create .= " primary key";
      }
      if ($field->referenceTable != "") {
        $create .= ", foreign key ($field->name)" . 
          " references $field->referenceTable";
        if ($field->referenceField != "") {
          $create .= " ($field->referenceField)";
        }
        if ($field->onDelete) {
          $create .= " on delete " . $field->onDelete;
        }
        if ($field->onUpdate != "") {
          $create .= " on update " . $field->onUpdate;
        }
      }
      if ($field->indexRegular) {
        $create .= ", index ($field->name";
        if ($field->indexSize > 0) {
          $create .= "($field->indexSize)";
        }
        $create .= ")";
      }
      
    }
    $create .= ") engine=innodb default character set utf8 collate utf8_general_ci;";
    return $create;
  }
    
}

?>